local natureMod = ...

natureMod:registerBehaviorTree({
	Id = MOD_NS .. "BEHAVIOR_FORESTER",
	VariableList = {
		{
			Name = "AgentData",
			DataType = "BEHAVIOR_TREE_DATA_AGENT",
			IsPublic = true,
			InitialValue = {}
		},
		{
			Name = "DoJobTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 7,
				Animation = AGENT_ANIMATION.FORESTER,
				SetIdleAfterWait = false
			}
		},
		{
			Name = "WorkplacePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true
			}
		},
        {
			Name = "TreePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = false,
				IsSetOrientationOnDestination = true
			}
		},
        {
			Name = "ShouldUseWorkplaceAnim",
			DataType = "BEHAVIOR_TREE_DATA_BOOL",
			IsPublic = false,
			InitialValue = {
				Value = true
			}
		},
        {
			Name = "ShouldReceiveXp",
			DataType = "BEHAVIOR_TREE_DATA_BOOL",
			IsPublic = false,
			InitialValue = {
				Value = true
			}
		},
		{
			Name = "WorkLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 1,
				Duration = 0,
				IsInfinite = false,
				IsDuration = false
			}
		},
        {
            Name = "PlantLoop",
            DataType = "BEHAVIOR_TREE_DATA_LOOP",
            IsPublic = true,
			InitialValue = {
				LoopCount = 7,
				Duration = 0,
				IsInfinite = false,
				IsDuration = false
			}
        },
        {
            Name = "PlantableData",
            DataType = "BEHAVIOR_TREE_DATA_PLANTABLE",
            IsPublic = true,
            InitialValue = {
            }
        }
	},
	Root = {
        Name = "WorkLoopRepeater",
        Type = "REPEAT",
        RepeatData = "WorkLoop",
        Child = {
            Name = "WorkLoopSequencer",
            Type = "SEQUENCER",
            Children = {
                {
                    Name = "HasWorkplace",
                    Type = "IS_WORKPLACE_AVAILABLE",
                    AgentData = "AgentData"
                },
                {
                    Name = "SetWorkplaceDestination",
                    Type = "SET_WORKPLACE_AS_DESTINATION",
                    AgentData = "AgentData",
                    WorkplacePosition = "WorkplacePosition",
                },
                {
                    Name = "GoToWorkplace",
                    Type = "GO_TO",
                    AgentData = "AgentData",
                    Destination = "WorkplacePosition",
                    BuildingPathType = "",
                    AnimationData = "",
                    AnimationSpeedMultiplier = ""
                },
                {
                    Name = "SetupForesterHut",
                    Type = MOD_NS .. "SETUP_FORESTER_HUT",
                    AgentData = "AgentData"
                },
                {
                    Name = "DisablePaths",
                    Type = "DISABLE_PATH_TRACING",
                    AgentData = "AgentData"
                },
                {
                    Name = "StartWorkShift",
                    Type = "START_WORK_SHIFT",
                    AgentData = "AgentData"
                },
                {
                    Name = "PlantingLoop",
                    Type = "REPEAT",
                    RepeatData = "PlantLoop",
                    Child = {
                        Name = "PlantSequencer",
                        Type = "SEQUENCER",
                        Children = {
                            {
                                Name = "FindNextTreePosition",
                                Type = MOD_NS .. "FIND_NEXT_PLANTING_POSITION",
                                AgentData = "AgentData",
                                TreePosition = "TreePosition",
                            },
                            -- {
                            --     Name = "FindNextTreePosition2",
                            --     Type = "FETCH_NEXT_PLANTABLE",
                            --     PlantPosition = "TreePosition",
                            --     PlantableData = "PlantableData"
                            -- },
                            {
                                Name = "GoToTreePosition",
                                Type = "GO_TO",
                                AgentData = "AgentData",
                                Destination = "TreePosition",
                                BuildingPathType = "",
                                AnimationData = "",
                                AnimationSpeedMultiplier = ""
                            },
                            {
                                Name = "PlantWaitTime",
                                Type = "WAIT",
                                AgentData = "AgentData",
                                TimeToWait = "DoJobTimer",
                                OptionalUseWorkstationAnim = "ShouldUseWorkplaceAnim"
                            },
                            {
                                Name = "PlantTree",
                                Type = MOD_NS .. "PLANT_TREE",
                                AgentData = "AgentData",
                                TreePosition = "TreePosition"
                            },
                        }
                    },
                },
                {
                    Name = "GoToWorkplace_2",
                    Type = "GO_TO",
                    AgentData = "AgentData",
                    Destination = "WorkplacePosition",
                    BuildingPathType = "",
                    AnimationData = "",
                    AnimationSpeedMultiplier = ""
                },
                {
                    Name = "GiveXp",
                    Type = "GIVE_JOB_XP",
                    AgentData = "AgentData",
                    ShouldReceiveXp = "ShouldReceiveXp"
                },
                {
                    Name = "FinishWorkShift",
                    Type = "FINISH_WORK_SHIFT",
                    AgentData = "AgentData"
                },
                {
                    Name = "EnablePaths",
                    Type = "ENABLE_PATH_TRACING",
                    AgentData = "AgentData"
                }
            }
        }
	}
})