local natureMod = ...

local TREES_PINE_A = MOD_NS .. "PREFAB_TREES_PINE_A"
local PINE_C = MOD_NS .. "PREFAB_PINE_A"
local PINE_B = MOD_NS .. "PREFAB_PINE_B"
local PINE_A = MOD_NS .. "PREFAB_PINE_C"
local FIR_C = MOD_NS .. "PREFAB_FIR_C"
local FIR_B = MOD_NS .. "PREFAB_FIR_B"
local FIR_A = MOD_NS .. "PREFAB_FIR_A"

local CONIFEROUS_TREES = {
    FIR_A, FIR_B, FIR_C, PINE_A, PINE_B, PINE_C, TREES_PINE_A
}

local DECIDUOUS_TREES = {
    MOD_NS .. "PREFAB_OAK_A",  MOD_NS .. "PREFAB_OAK_B", MOD_NS .. "PREFAB_OAK_C", MOD_NS .. "PREFAB_OAK_SAPLING_A",  MOD_NS .. "PREFAB_TREES_OAK_A",
    MOD_NS .. "PREFAB_BEECH_A", MOD_NS .. "PREFAB_BEECH_B", MOD_NS .. "PREFAB_BEECH_C", MOD_NS .. "PREFAB_BEECH_D", MOD_NS .. "PREFAB_BEECH_SAPLING_A",
    MOD_NS .. "PREFAB_MAPLE_A", MOD_NS .. "PREFAB_MAPLE_B", MOD_NS .. "PREFAB_BIRCH_A", MOD_NS .. "PREFAB_BIRCH_B", MOD_NS .. "PREFAB_BIRCH_C"
}

local ALL_TREES = { FIR_A, FIR_B, FIR_C, PINE_A, PINE_B, PINE_C, TREES_PINE_A,
    MOD_NS .. "PREFAB_OAK_A",  MOD_NS .. "PREFAB_OAK_B", MOD_NS .. "PREFAB_OAK_C", MOD_NS .. "PREFAB_OAK_SAPLING_A",  MOD_NS .. "PREFAB_TREES_OAK_A",
    MOD_NS .. "PREFAB_BEECH_A", MOD_NS .. "PREFAB_BEECH_B", MOD_NS .. "PREFAB_BEECH_C", MOD_NS .. "PREFAB_BEECH_D", MOD_NS .. "PREFAB_BEECH_SAPLING_A",
    MOD_NS .. "PREFAB_MAPLE_A", MOD_NS .. "PREFAB_MAPLE_B", MOD_NS .. "PREFAB_BIRCH_A", MOD_NS .. "PREFAB_BIRCH_B", MOD_NS .. "PREFAB_BIRCH_C"
}

local FOREST_HUT_COMPONENT = {
	TypeName = FOREST_HUT_COMP_TYPE,
	ParentType = "COMPONENT",
    Properties = {
        { Name = "densityMap", Type = "list<COMPONENT>", Default = {}, Flags = { "SAVE_GAME" }},
        { Name = "averageTreeDistance", Type = "integer", Default = 7 },
    }
}

function FOREST_HUT_COMPONENT:plantConiferousTrees()
    self.treePrefabs = CONIFEROUS_TREES
end

function FOREST_HUT_COMPONENT:plantDeciduousTrees()
    self.treePrefabs = DECIDUOUS_TREES
end

function FOREST_HUT_COMPONENT:plantAllTrees()
    self.treePrefabs = ALL_TREES
end

function FOREST_HUT_COMPONENT:init()
    self.seeds = {}
    self.treePrefabs = CONIFEROUS_TREES
end

function FOREST_HUT_COMPONENT:removeSeed(seed)
    self.seeds[seed.id] = nil
end

function FOREST_HUT_COMPONENT:addSeed(seed)
    self.seeds[seed.id] = seed
end

function FOREST_HUT_COMPONENT:getNextSeedPosition()
    local key,seed = next(self.seeds, nil)

    if seed == nil then
        return nil
    end

    self.seeds[key] = nil

    local plant = seed:getOwner()
    local position = plant:getGlobalPosition()
    plant:destroy()

    return position
end

function FOREST_HUT_COMPONENT:plantATree(position)
    local hash = math.floor(position[1] / self.averageTreeDistance) .. "-" .. math.floor(position[3] / self.averageTreeDistance)
    if self.densityMap[hash] ~= nil and
        pcall(function()
                self.densityMap[hash]:getOwner()
            end) then
        return false
    end

    local orientation = { 0, 0, 0, 1 }
    quaternion.setEulerAngles(orientation, { 0, math.random()*360, 0 }) 
    local treeIndex = math.random(1, #self.treePrefabs)
    local tree = self:getLevel():createObject(self.treePrefabs[treeIndex], position, orientation, function(obj)
        obj:setScale(0.1)
        obj:getOrCreateComponent(GROWTH_CONTROLLER_GROWTH_TYPE)
        local resourceContainer = obj:getOrCreateComponent("COMP_RESOURCE_CONTAINER")
        resourceContainer.ResourceValue = 0
        resourceContainer.AvailableQuantity = 0
    end)

    self.densityMap[hash] = tree:getOrCreateComponent("COMP_TREE")
    return true
end

natureMod:registerClass(FOREST_HUT_COMPONENT)
